"""Functional tests for etckeeper charm."""
import os
import pytest
import subprocess
import stat

# Treat all tests as coroutines
pytestmark = pytest.mark.asyncio

charm_build_dir = os.getenv("CHARM_BUILD_DIR", ".").rstrip("/")
series = [
    "xenial",
    "bionic",
    "focal",
]
sources = [
    ("local", "{}/etckeeper".format(charm_build_dir)),
    # ('jujucharms', 'cs:...'),
]


# Custom fixtures
@pytest.fixture(params=series)
def series(request):
    """Allow testing of each series."""
    return request.param


@pytest.fixture(params=sources, ids=[s[0] for s in sources])
def source(request):
    """Allow testing of each install method (charmstore vs local)."""
    return request.param


@pytest.fixture
async def app(model, series, source):
    """Allow testing of each instance of this charm in the model."""
    app_name = "etckeeper-{}-{}".format(series, source[0])
    return await model._wait_for_new("application", app_name)


async def test_charmetckeeper_deploy(model, series, source, request):
    """Test deploying the charm and awaiting status."""
    # Starts a deploy for each series
    # Using subprocess b/c libjuju fails with JAAS
    # https://github.com/juju/python-libjuju/issues/221
    application_name = "etckeeper-{}-{}".format(series, source[0])
    cmd = [
        "juju",
        "deploy",
        source[1],
        "-m",
        model.info.name,
        "--series",
        series,
        application_name,
    ]
    if request.node.get_closest_marker("xfail"):
        # If series is 'xfail' force install to allow testing against versions not in
        # metadata.yaml
        cmd.append("--force")
    subprocess.check_call(cmd)


async def test_ubuntu_deploy(model, series, request):
    """Test deploying the charm and awaiting status."""
    # Starts a deploy for each series
    # Using subprocess b/c libjuju fails with JAAS
    # https://github.com/juju/python-libjuju/issues/221
    application_name = "ubuntu-{}".format(series)
    cmd = [
        "juju",
        "deploy",
        "cs:ubuntu",
        "-m",
        model.info.name,
        "--series",
        series,
        application_name,
    ]
    if request.node.get_closest_marker("xfail"):
        # If series is 'xfail' force install to allow testing against versions not in
        # metadata.yaml
        cmd.append("--force")
    subprocess.check_call(cmd)


async def test_etckeeper_relate(model, series, app, request):
    """Test relating etckeeper subordinte to ubuntu."""
    application_name = "ubuntu-{}".format(series)
    ubuntu = model.applications[application_name]
    await model.block_until(lambda: ubuntu.status == "active" or ubuntu.status == "error")
    await model.add_relation(app.name, application_name)


async def test_charmetckeeper_status(model, app, series):
    """Test the status of the application in the model."""
    application_name = "ubuntu-{}".format(series)
    ubuntu = model.applications[application_name]
    await model.block_until(lambda: ubuntu.status == "active" or ubuntu.status == "error")
    await model.block_until(lambda: app.status == "active" or app.status == "error")
    assert ubuntu.status != "error"
    assert app.status != "error"


async def test_charm_upgrade(model, app):
    """Test upgrading the charm from the charmstore version."""
    if app.name.endswith("local"):
        pytest.skip("No need to upgrade the local deploy")
    unit = app.units[0]
    await model.block_until(lambda: unit.agent_status == "idle")
    subprocess.check_call(
        [
            "juju",
            "upgrade-charm",
            "--switch={}".format(sources[0][1]),
            "-m",
            model.info.name,
            app.name,
        ]
    )
    await model.block_until(lambda: unit.agent_status == "executing")


async def test_charmetckeeper_status_postupgrade(model, app):
    """Test the status of the application in the model."""
    await model.block_until(lambda: app.status == "active")
    unit = app.units[0]
    await model.block_until(lambda: unit.agent_status == "idle")


async def test_commit_action(app):
    """Test running the commit action."""
    unit = app.units[0]
    action = await unit.run_action("commit")
    action = await action.wait()
    assert action.status == "completed"


async def test_run_command(app, jujutools):
    """Test running a command with a known output on the unit."""
    unit = app.units[0]
    cmd = "hostname -i"
    results = await jujutools.run_command(cmd, unit)
    assert results["Code"] == "0"
    assert unit.public_address in results["Stdout"]


async def test_file_stat(app, jujutools):
    """Test the contents of a known file on the unit."""
    unit = app.units[0]
    path = "/var/lib/juju/agents/unit-{}/charm/metadata.yaml".format(
        unit.entity_id.replace("/", "-")
    )
    fstat = await jujutools.file_stat(path, unit)
    assert stat.filemode(fstat.st_mode) == "-rw-r--r--"
    assert fstat.st_uid == 0
    assert fstat.st_gid == 0


async def test_repo_stat(app, jujutools):
    """Test the contents of the git repo."""
    unit = app.units[0]
    fstat = await jujutools.file_stat("/etc/.git/index", unit)
    assert stat.filemode(fstat.st_mode) == "-rw-r--r--"
    assert fstat.st_uid == 0
    assert fstat.st_gid == 0
